#!/usr/bin/python
# Author: Brian Murray <brian@canonical.com>
# Copyright (C) 2010 Canonical, Ltd.
# License: GPLv3

from datetime import datetime
from lpltk import LaunchpadService

import optparse
import simplejson as json
import sys


OPEN_STATUS = ['New', 'Incomplete', 'Confirmed', 'Triaged', 'In Progress',
               'Fix Committed']
CLOSED_STATUS = ['''Won't Fix''', 'Fix Released', 'Invalid', 'Expired',
                 'Opinion']
ALL_STATUS = OPEN_STATUS + CLOSED_STATUS


def find_hottest_bugs(package):
    bugs  = []
    tasks = package.searchTasks(status=OPEN_STATUS, order_by='-heat')
    for task in tasks[:5]:
        bugs.append(task.bug)
    return bugs


def create_json_file(filename, data):
    json_file = open(filename, 'w')
    json_file.write(json.dumps(data, indent=4))
    json_file.close()


def jsonize_hot_bugs(package, hot_bugs):
    package_data = {}
    package_data['package'] = package.name
    index = 1
    for bug in hot_bugs:
        package_data['heat%s' % index] = {'bug': bug.id,
            'heat': bug.heat,
            'title': bug.title}
        index += 1
    return package_data

if __name__ == '__main__':
    lp = LaunchpadService(config=
        {'read_only': False,
         'display_name': 'defect-analysts-hot-bugs-json'})

    ubuntu = lp.load_project('ubuntu')

    parser = optparse.OptionParser()
    parser.add_option("--team", help="Team name")
    parser.add_option("--packagesets", default=False, action="store_true")

    (opt, args) = parser.parse_args()
    if opt.team and opt.packagesets:
        print("One must choose either packagesets or team, not both")
        sys.exit(1)

    if opt.team:
        team = lp.launchpad.people[opt.team]

        data = []
        run_time = datetime.strftime(datetime.now(), '%Y-%m-%d %H:%M')
        packages = team.getBugSubscriberPackages()
        json_filename = '%s-package-bug-heats.json' % (opt.team)
        for package in packages:
            hot_bugs = find_hottest_bugs(package)
            if len(hot_bugs) == 0:
                continue
            #package_data = {}
            #package_data['package'] = package.name
            #index = 1
            #for bug in hot_bugs:
            #    package_data['heat%s' % index] = {'bug': bug.id,
            #        'heat': bug.heat,
            #        'title': bug.title}
            #    index += 1
            package_data = jsonize_hot_bugs(package, hot_bugs)
            data.append(package_data)
        data.append({'ran': run_time})

        create_json_file(json_filename, data)

    if opt.packagesets:
        run_time = datetime.strftime(datetime.now(), '%Y-%m-%d %H:%M')
        dev_release = ubuntu.current_series
        package_sets = [ps.name for ps in lp.launchpad.packagesets if ps.distroseries == dev_release]
        for package_set in package_sets:
            data = []
            json_filename = "%s-package-bug-heats.json" % (package_set)
            packages = lp.launchpad.packagesets.getByName(distroseries=dev_release,
                name=package_set).getSourcesIncluded()
            for pkg_name in packages:
                src_pkg = ubuntu.getSourcePackage(name=pkg_name)
                hot_bugs = find_hottest_bugs(src_pkg)
                if len(hot_bugs) == 0:
                    continue
                package_data = jsonize_hot_bugs(src_pkg, hot_bugs)
                data.append(package_data)
            data.append({'ran': run_time})

            create_json_file(json_filename, data)
