#!/usr/bin/python

from lpltk import LaunchpadService
from mako.template import Template


def create_report(content, report_name):
    report = open(report_name, 'w')
    report.write(content)
    report.close()


if __name__ == '__main__':
    pkg_tasks_template = Template(filename="recent-package-bug-tasks.mako")
    duplicates_template = Template(filename="bugs-with-recent-duplicates.mako")
    hottest_bugs_template = Template(filename="package-bug-heats.mako")
    # teams are statically defined at the moment
    teams = {'foundations-bugs': 'Ubuntu Foundations',
        'ubuntu-server': 'Ubuntu Server',
        'desktop-packages': 'Ubuntu Desktop',
        'ubuntu-x-swat': 'Ubuntu X Swat'}
    for team in teams:
        explanation = 'to which the %s team are subscribed' % teams[team]
        pkg_tasks_report_fname = '%s-recent-bug-tasks.html' % team
        content = pkg_tasks_template.render(team_name=team,
            team_propername=teams[team],
            explanation=explanation)
        create_report(content, pkg_tasks_report_fname)
        dupes_report_fname = '%s-bugs-with-recent-dupes.html' % team
        content = duplicates_template.render(team_name=team,
            team_propername=teams[team],
            days='14')
        create_report(content, dupes_report_fname)
        hot_report_fname = '%s-packages-hottest-bugs.html' % team
        content = hottest_bugs_template.render(team_name=team,
            team_propername=teams[team],
            explanation=explanation)
        create_report(content, hot_report_fname)
    # lookup packagesets in Launchpad - maybe this should be cached
    lp = LaunchpadService(config =
        {'read_only': True,
         'display_name': 'defect-analysts-graph-reports'})
    ubuntu = lp.load_project('ubuntu')
    dev_release = ubuntu.current_series
    package_sets = [ps.name for ps in lp.launchpad.packagesets if ps.distroseries == dev_release]
    for package_set in package_sets:
        explanation = 'that make up the %s package set' % package_set
        pkg_tasks_report_fname = '%s-recent-bug-tasks.html' % package_set
        content = pkg_tasks_template.render(team_name=package_set,
            team_propername=package_set,
            explanation=explanation)
        create_report(content, pkg_tasks_report_fname)
        hot_report_fname = '%s-packages-hottest-bugs.html' % package_set
        content = hottest_bugs_template.render(team_name=package_set,
            team_propername=package_set,
            explanation=explanation)
        create_report(content, hot_report_fname)
