#!/usr/bin/python
# Author: Brian Murray <brian@canonical.com>
# Copyright (C) 2012 Canonical, Ltd.
# License: GPLv3

# For a given Ubuntu package calculate the median bug gravity and return the
# gravity of all the bug reports

from operator import itemgetter
import lpl_common
import sys
import numpy

lp = lpl_common.connect(use_edge=False, version='devel')

ubuntu = lp.distributions['ubuntu']

# 2012-01-12 this doesn't recurse the teams in bugcontrol (I think)
bugcontrol_members = [p.name for p in lp.people['ubuntu-bugcontrol'].participants]
udev_members = [p.name for p in lp.people['ubuntu-dev'].participants]

package_name = sys.argv[1]

pkg_median_gravitys = []
package = ubuntu.getSourcePackage(name=package_name)
pkg_bugs = [task.bug for task in package.searchTasks()]

pkg_gravity = 0
gravities = []
occurrences = {}
bugs = {}

tag_weights = {'apport-bug': 50, 'apport-package': 100, 'apport-crash': 100,
               'apport-kerneloops': 150, 'regression-release': 200,
               'regression-proposed': 250, 'regresion-updates': 300,
               'precise': 200, 'bugpattern-written': 300,
               'bugpattern-needed': 300}

weighted_tags = set(tag for tag in tag_weights)

for bug in pkg_bugs:
    bug_gravity = bug.number_of_duplicates * 6
    # N.B. values for affected vs subs are switched compared to what it used
    # to be when heat math was done
    bug_gravity += bug.users_affected_count * 4
    bug_gravity += len(bug.subscriptions) * 2
    bug_tags = set(tag for tag in bug.tags)
    if bug_tags.intersection(weighted_tags):
        for tag in bug_tags.intersection(weighted_tags):
            bug_gravity += tag_weights[tag]
    # increase gravity if the reporter is in ubuntu-dev or bugcontrol
    reporter = bug.owner.name
    if reporter in bugcontrol_members:
        bug_gravity += 150
    if reporter in udev_members:
        bug_gravity += 200
    # 2012-01-12 ++gravity if the bug has a patch or if it has a bug watch
    # that is fixed?
    pkg_gravity += bug_gravity
    gravities.append(bug_gravity)
    bugs[bug.id] = bug_gravity

std = numpy.std(gravities)

print "%s average gravity: %i" % (package_name, std)

bugs = sorted(bugs.items(), key=itemgetter(1), reverse=True)
for bug in bugs:
    bg = bug[1]
    print "LP: #%s - %s (%s)" % (bug[0], bg, (bg - std))
