#!/usr/bin/python
# Author: Brian Murray <brian@canonical.com>
# Copyright (C) 2010 Canonical, Ltd.
# License: GPLv3
#
# given a team name retrieve the recent-package-bugs json for that team and
# compare bug reporting volume on a per package basis for the periods defined
# in that data file producing information like the following:
#
# ubiquity received 44 bugs (21 dupes) this period compared to 27 last period
#   LP: #1020848 (Ultimate Linux 3.4 crashed on installation, see machine generated Bug report attached. ) received 6 dupes
#   LP: #1020574 (SystemError: Broken pipe while installing language packs) received 13 dupes
#
# This should show you which packages are receiving spikes in bug volume and
# which bugs are receiving duplicate reports

from lpltk import LaunchpadService
from urllib2 import urlopen

import optparse
import simplejson as json
import sys


if __name__ == '__main__':
    launchpad = LaunchpadService(config =
        {'read_only': False,
         'display_name': 'defect-analysts-spike'})

    parser = optparse.OptionParser(usage="usage: %prog --team TEAM")

    parser.add_option('--team', help='team to check')

    (opt, args) = parser.parse_args()

    if opt.team:
        team_name = opt.team
    else:
        print('A team name must be provided.')
        sys.exit(1)

    datafile = urlopen('http://reports.qa.ubuntu.com/reports/%s/%s-recent-package-bugs.json' % (team_name, team_name))
    json_data = json.load(datafile)

    period1 = {}
    period2 = {}
    periods = []

    for data in json_data:
        if 'ran' in data.keys():
            continue
        if data['period'] not in periods:
            periods.append(data['period'])

    periods.sort()

    for data in json_data:
        if 'ran' in data.keys():
            continue
        if data['period'] == periods[0]:
            period1[data['package']] = {'total': data['total'], 'dupes':
                data['dupes'], 'masters': data['masters']}
        if data['period'] == periods[1]:
            period2[data['package']] = {'total': data['total'], 'dupes':
                data['dupes'], 'masters': data['masters']}

    for package, count in sorted(period1.items()):
        if package in period2 and 'total' in period2[package]:
            last = period2[package]['total']
            # maybe look at the volume of total bugs received?
            if last == 1 and count['total'] <= 4:
                continue
            if count['total'] > round(last * 1.1):
                print('%s received %s bugs (%s dupes) this period compared to %s last period'
                    % (package, count['total'], count['dupes'], last))
                print('  https://bugs.launchpad.net/ubuntu/+source/%s/+bugs?orderby=-id' % package)
                if count['dupes'] > 0:
                    for k in count['masters']:
                        if count['masters'][k] >= 2:
                            try:
                                bug = launchpad.get_bug(k)
                                dupecount = len(bug.lpbug.duplicates)
                                # we only need to check the reporters of the
                                # recent duplicates, not all of them
                                reporters = set([d.owner.name for d in bug.lpbug.duplicates[dupecount-count['masters'][k]:dupecount]])
                            except KeyError:
                                print("  LP: #%s is not accesible but received %s dupes?" % (k, count['masters'][k]))
                                reporters = []
                            # check to see if the new duplicate bugs are from the same reporter
                            if len(reporters) < 2:
                                continue
                            print('  LP: #%s (%s) received %s dupes' % (k,
                                bug.title, count['masters'][k]))
        else:
            if count['total'] >= 3:
                print('%s received %s bugs (%s dupes) this period compared to 0 last period'
                    % (package, count['total'], count['dupes']))
                print('  https://bugs.launchpad.net/ubuntu/+source/%s/+bugs?orderby=-id' % package)
                if count['dupes'] > 0:
                    for k in count['masters']:
                        if count['masters'][k] >= 2:
                            bug = launchpad.get_bug(k)
                            print('  LP: #%s (%s) received %s dupes' % (k,
                                bug.title, count['masters'][k]))
