#!/usr/bin/python3

import sys
import logging

CURRENT_UBUNTU_STABLE = 'jammy'
CURRENT_UBUNTU_DEV = 'lunar'

# map openstack releases to ubuntu releases
VERSION_MAP = {
    'folsom': {
        'ubuntu_stable': 'precise',
        'ubuntu_dev': 'quantal',
    },
    'grizzly': {
        'ubuntu_stable': 'precise',
        'ubuntu_dev': 'raring',
    },
    'havana': {
        'ubuntu_stable': 'precise',
        'ubuntu_dev': 'saucy',
    },
    'icehouse': {
        'ubuntu_stable': 'precise',
        'ubuntu_dev': 'trusty',
    },
    'juno': {
        'ubuntu_stable': 'trusty',
        'ubuntu_dev': 'utopic',
    },
    'kilo': {
        'ubuntu_stable': 'trusty',
        'ubuntu_dev': 'vivid',
    },
    'liberty': {
        'ubuntu_stable': 'trusty',
        'ubuntu_dev': 'wily',
    },
    'mitaka': {
        'ubuntu_stable': 'trusty',
        'ubuntu_dev': 'xenial',
    },
    'newton': {
        'ubuntu_stable': 'xenial',
        'ubuntu_dev': 'yakkety',
    },
    'ocata': {
        'ubuntu_stable': 'xenial',
        'ubuntu_dev': 'zesty',
    },
    'pike': {
        'ubuntu_stable': 'xenial',
        'ubuntu_dev': 'artful',
    },
    'queens': {
        'ubuntu_stable': 'xenial',
        'ubuntu_dev': 'bionic',
    },
    'rocky': {
        'ubuntu_stable': 'bionic',
        'ubuntu_dev': 'cosmic',
    },
    'stein': {
        'ubuntu_stable': 'bionic',
        'ubuntu_dev': 'disco',
    },
    'train': {
        'ubuntu_stable': 'bionic',
        'ubuntu_dev': 'eoan',
    },
    'ussuri': {
        'ubuntu_stable': 'bionic',
        'ubuntu_dev': 'focal',
    },
    'victoria': {
        'ubuntu_stable': 'focal',
        'ubuntu_dev': 'groovy',
    },
    'wallaby': {
        'ubuntu_stable': 'focal',
        'ubuntu_dev': 'hirsute',
    },
    'xena': {
        'ubuntu_stable': 'focal',
        'ubuntu_dev': 'impish',
    },
    'yoga': {
        'ubuntu_stable': 'focal',
        'ubuntu_dev': 'jammy',
    },
    'zed': {
        'ubuntu_stable': 'jammy',
        'ubuntu_dev': 'kinetic',
    },
    'antelope': {
        'ubuntu_stable': 'jammy',
        'ubuntu_dev': 'lunar',
    },
    'bobcat': {
        'ubuntu_stable': 'jammy',
        'ubuntu_dev': 'mantic',
    },
    'cloud-tools': {
        'ubuntu_stable': CURRENT_UBUNTU_STABLE,
        'ubuntu_dev': 'trusty',
    },
    'cloud-tools-next': {
        'ubuntu_stable': CURRENT_UBUNTU_STABLE,
        'ubuntu_dev': 'trusty',
        'staging_ppa': 'cloud-tools-next',
        'ca_pockets': [],
    }
}

CA_PACKAGES_URL = ("http://ubuntu-cloud.archive.canonical.com/ubuntu/dists/"
                   "%s-%s/%s/main/source/Sources.gz")


SUPPORTED_OS_RELEASES = ['ussuri', 'wallaby', 'yoga', 'zed', 'antelope']
DEV_OS_RELEASES = ['bobcat']

def packages_url(os_release, pocket):
    ubuntu_stable = VERSION_MAP[os_release]['ubuntu_stable']

    try:
        url = CA_PACKAGES_URL % (ubuntu_stable,
                                 pocket,
                                 os_release)
        return url
    except:
        logging.error('Couldnt determine packages URL.')
        sys.exit(1)
