#!/usr/bin/python

import argparse
import errno
import urllib
import re
import apt_pkg
import urllib2
import sys
from debian.changelog import Changelog
import os
import datetime

from launchpadlib.launchpad import Launchpad


class URLCache(object):
    def __init__(self, path, save_used=False):
        self.path = path
        if save_used:
            self.used = set()
        else:
            self.used = None

        if not os.path.exists(self.path):
            os.mkdir(self.path)

    def note_used(self, filename):
        if self.used is not None:
            self.used.add(filename)

    def open(self, filename, mode='r'):
        file_path = os.path.join(self.path, filename)
        f = open(file_path, mode)
        self.note_used(filename)
        return f

    def open_or_fetch(self, filename, url):
        try:
            return self.open(filename)
        except IOError, e:
            if e.errno != errno.ENOENT:
                raise
            disk_file = self.open(filename, mode='w+')
            http_file = urllib2.urlopen(url)
            disk_file.write(http_file.read())
            http_file.close()
            disk_file.seek(0)
            return disk_file

    def unlink_unused(self):
        if self.used is None:
            return

        for filename in os.listdir(self.path):
            if filename in self.used:
                continue
            file_path = os.path.join(self.path, filename)
            if os.path.isfile(file_path):
                os.unlink(file_path)


def checksyncormerge(lp,srcpkg):
    # Fetch the package's bug list from Launchpad
    pkg = lp.distributions['ubuntu'].getSourcePackage(name=srcpkg)
    pkgBugList = pkg.searchTasks(omit_duplicates=True)

    # Search bug list for other sync requests.
    for bug in pkgBugList:
        # check for Sync or sync and the package name
        if not bug.is_complete and ('ync %s' % srcpkg in bug.title or 'erge %s' % srcpkg in bug.title):
            #print >> sys.stderr, ('BUG: %s' % bug)
            return bug
    return False



def get_package_list(lp, pkgs):
	p = re.compile('>([^\s]+)\sin\sUbuntu')
	team_name = 'ubuntu-server'
	team = lp.people[team_name]
	(f_name, h) = urllib.urlretrieve(\
		"https://bugs.launchpad.net/~%s/+packagebugs" % team.name)

	for l in open(f_name):
		m = p.search(l)
		if m:
			pkgs.append(m.group(1))

	return pkgs

def get_package_set(lp, series, packageset):
        archives = lp.distributions['ubuntu'].archives
        lp_series = lp.distributions['ubuntu'].getSeries(
            name_or_version=series)
        lp_set = lp.packagesets.getByName(name=packageset,
                                                     distroseries=lp_series)

        perms = []
        for archive in archives:
                perms.extend(archive.getUploadersForPackageset(
                    packageset=lp_set))

        sources = sorted(lp_set.getSourcesIncluded(direct_inclusion=True))
        return sources
        #if sources:
        #        print
        #        print ("== All source packages in package set '%s' "
        #               "in '%s' ==" % (packageset, series))
        #        for source in sources:
        #            print source
        #child_sets = list(lp_set.setsIncluded(direct_inclusion=True))
        #if child_sets:
        #        print
        #        print ("== All package sets in package set '%s' in '%s' ==" %
        #               (packageset, series))
        #        for child_set in child_sets:
        #            print child_set.name



def get_ubuntu_list(lp, series, package, returnobj=False):
	ubuntu = lp.distributions['ubuntu']
	
	primary, partner = ubuntu.archives
	releases = ubuntu.getSeries(name_or_version=series)
	
	try:
                def get_source_package_publishing_history(pocket):
		    return primary.getPublishedSources(
                      exact_match=True, 
                      source_name=package, 
                      pocket=pocket, 
                      distro_series=releases)

                spph = get_source_package_publishing_history('Proposed')
                if not spph:
                    spph = get_source_package_publishing_history('Release')
                if spph:
                    upload_version = spph[0]
                else:
                    raise

                if returnobj == False:
                  upload_version = upload_version.source_package_version
	except:
		upload_version = "n/a"

	return upload_version

def get_debian_package(lp, package):
	debian = lp.distributions['debian']

	primary = debian.main_archive
	releases = debian.getSeries(name_or_version='sid')

	try:
		upload_debian = primary.getPublishedSources(
                  exact_match=True, 
                  source_name=package, 
                  pocket="Release", 
                  distro_series=releases)[0]
	except:
		upload_debian = 0

	return upload_debian

def get_ubuntu_uploader(lp, package):
                     i = package
                     uploader = " * Last Uploader: "
                     try:
                        uploader += "%s" % (i.package_creator.display_name.encode('utf-8'))
                        if i.package_creator != i.package_signer:
                            uploader += " (sponsored by %s)" % (i.package_signer.display_name.encode('utf-8'))
                     except:
                        uploader = "Unknown (Probably auto sync.)"
                     return uploader

def get_debian_list(lp, package):
  debian_pkg = get_debian_package(lp,package)
  if debian_pkg:
    upload_version = debian_pkg.source_package_version
  else:
    upload_version = 0
  return upload_version

def debian_newer(ubuntu,debian):
  if debian == 0:
    return False
  apt_pkg.init_system()
  delta = apt_pkg.version_compare(ubuntu, debian)
  if delta < 0:
    return True
  else:
    return False

def getDebianChangelog(srcpkg, version, url_cache):
    '''
    Return the new changelog entries upto 'version'.
    -- Based on the function from requestsync
    '''
    pkgname = srcpkg.source_package_name
    pkgversion = srcpkg.source_package_version
    component = srcpkg.component_name
    if pkgname.startswith('lib'):
        subdir = 'lib%s' % pkgname[3]
    else:
        subdir = pkgname[0]
    # Strip epoch from version
    if ':' in pkgversion:
        pkgversion = pkgversion[pkgversion.find(':')+1:]

    changelog_file = "%s_%s.txt" % (pkgname,pkgversion)
    changelog_url = ('http://ftp-master.metadata.debian.org/changelogs/'
                                '/%s/%s/%s/%s_%s_changelog'
                                % (component, subdir, pkgname, pkgname,
                                   pkgversion))

    f = url_cache.open_or_fetch(changelog_file, changelog_url)
    changelog = f.read()
    f.close()

    new_entries = ''
    changelog = Changelog(changelog)
    for block in changelog:
      if block.package == pkgname:
        if pkgname == 'apache2':
            return "Error creating changes log. Please look manually"
        if block.version > version:
                # see also Debian #561805
                new_entries += unicode(str(block).decode('utf-8'))

    return new_entries


def main(lp_cache_dir, changelog_cache_dir, output_file):
        print >>output_file, """<html>
<head>
<meta http-equiv="content-type" content="text/html; charset=utf-8">
<style>body { font-family: Ubuntu, sans-serif; }</style>
<link rel="stylesheet" type="text/css" href="http://fonts.googleapis.com/css?family=Ubuntu:regular,bold&subset=Latin">
</head>
<body>"""
        print >>output_file, "<p>Delta generated at: %s UTC</p>" % (datetime.datetime.utcnow().strftime("%Y-%m-%d %H:%M"))

        url_cache = URLCache(changelog_cache_dir, save_used=True)
	pkgs = []
        lp = Launchpad.login_anonymously(
            'ubuntu-debian-delta', "production", lp_cache_dir)
        dev_series = lp.distributions['ubuntu'].current_series.name
        packageset = get_package_set(lp, series=dev_series, packageset="ubuntu-server")
        subscribedpkgs = get_package_list(lp, pkgs)
        pkgs = sorted(list(set(list(subscribedpkgs)+list(packageset))))
	for i in pkgs:
		ubuntu_upload_version = get_ubuntu_list(lp, dev_series, i)
		debian_upload_version = get_debian_list(lp, i)
                ubuntupkgurl = "https://launchpad.net/ubuntu/+source/%s/%s" % (i,ubuntu_upload_version,)
                if debian_newer(ubuntu_upload_version,debian_upload_version):
		  print >>output_file, "<h3> %-10s <a href='%-10s'>%-10s<a> -> %-10s </h3>" %(i, ubuntupkgurl, ubuntu_upload_version, debian_upload_version)
                  ubuntu_uploader = get_ubuntu_uploader(lp, get_ubuntu_list(lp, dev_series, i, returnobj=True ))
                  print >>output_file, "<p>%s</p>" % ubuntu_uploader
                  bug = checksyncormerge(lp,i)
                  if bug:
                    print >>output_file, "<p>Sync or Merge bug: <a href='%s'>%s</a> (%s)</p>" % (bug.web_link,bug.title,bug.status)
                  print >>output_file, "<p>Debian changes newer than ubuntu version:</p>"
                  debian_changelog = getDebianChangelog(get_debian_package(lp,i),ubuntu_upload_version, url_cache).encode('utf-8')
                  print >>output_file, "<pre>%s</pre>" % debian_changelog
        print >>output_file, "</body></html>"
        url_cache.unlink_unused()

def mkdir_p(path):
    """Create path if it doesn't exist already."""
    try:
        os.makedirs(path)
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--cache-dir', default='cache')
    parser.add_argument('--output-file', '-o', type=argparse.FileType('w'),
        default=sys.stdout)
    args = parser.parse_args()
    lp_cache_dir = os.path.join(args.cache_dir, 'launchpadlib')
    changelog_cache_dir = os.path.join(args.cache_dir, 'changelogs')
    mkdir_p(lp_cache_dir)
    mkdir_p(changelog_cache_dir)
    main(lp_cache_dir, changelog_cache_dir, args.output_file)
