#!/usr/bin/env python
from __future__ import with_statement

from launchpadlib.launchpad import Launchpad
from lp_utils import BugTask
import re
from os import path
from datetime import datetime
import reporting
import pdb


launchpad = Launchpad.login_anonymously('server-release-bugs', 'production')
#launchpad = Launchpad.login_with('server-release-bugs', 'production')
ubuntu = launchpad.projects['ubuntu']

team = launchpad.people["canonical-server"].members_details
SUPERVISOR = launchpad.people["ubuntu-server"]

UBUNTU_SERVER_TRACKING = launchpad.people["ubuntu-server-release-tracking"]
UBUNTU_SERVER_TRACKING_OTHER = launchpad.people["ubuntu-server-release-tracking-other"]

UBUNTU_MIR_TEAM = launchpad.people["ubuntu-mir"]

def inteam(person):
    for member in team:
        if member.member == person:
            return True
    return False

def trackingbyteam(supervisor=None,subscriber=None):
    bugs = []
    rsbugtasks = ubuntu.searchTasks(bug_subscriber=subscriber,bug_supervisor=supervisor)
    for bugtask in rsbugtasks:
        bugs.append(BugTask(bugtask))
    return bugs

def fortag(tag,supervisor=None):
    bugs = []
    if supervisor:
        rsbugtasks = ubuntu.searchTasks(tags=tag,bug_supervisor=supervisor)
    else:
        rsbugtasks = ubuntu.searchTasks(tags=tag)
    for bugtask in rsbugtasks:
        if tag == "server-q-ro" and inteam(bugtask.assignee):
            continue
        bugs.append(BugTask(bugtask))
    return bugs

report_timestamp = datetime.utcnow().strftime("%Y-%m-%d %H:%M")
server_release_bugs = trackingbyteam(subscriber=UBUNTU_SERVER_TRACKING)
otherteams_release_bugs = trackingbyteam(subscriber=UBUNTU_SERVER_TRACKING_OTHER)
ftbfs_bugs = fortag('ftbfs', SUPERVISOR)
mir_bugs = trackingbyteam(supervisor=SUPERVISOR,subscriber=UBUNTU_MIR_TEAM)
ffe_bugs = fortag("server-q-ffe")
context = dict(
    report_timestamp=report_timestamp,
    server_release_bugs=server_release_bugs,
    otherteams_release_bugs=otherteams_release_bugs,
    ftbfs_bugs=ftbfs_bugs,
    mir_bugs=mir_bugs,
    ffe_bugs=ffe_bugs,
    )

directory = "/srv/qa.ubuntu.com/reports/ubuntu-server/"
report_name = "release-bugs.html"
report_path = path.join(directory, report_name)
with file(report_path, 'w') as reportfile:
    reporting.render_deployment_report(reportfile, context,
        'release-bugs-table.html')

data = {("Bugs pertinent to Quantal Server release that we are working on "
        "(currently %d)" % len(server_release_bugs)) : server_release_bugs,
        ("Bugs we are tracking that other teams are working on (currently %s)"
            % len(otherteams_release_bugs)) : otherteams_release_bugs,
        ("FTBFS that we need to look at (currently %s)" %
            len(ftbfs_bugs)) : ftbfs_bugs,
        ("MIR's we are tracking required for Precise Release (currently %s)" %
            len(mir_bugs)) : mir_bugs,
        ("Feature Freeze exceptions that are required for release (currently"
            " %s)" % len(ffe_bugs)) : ffe_bugs,
        }

report_name = 'release-bugs.txt'
report_path = path.join(directory, report_name)

reporting.write_wiki_output(data, report_path)
