#!/usr/bin/python

from lpltk import LaunchpadService

import optparse
import simplejson as json


def process_tasks(hot_tasks):
    #bugs = {number: title, ....}
    bugs = {}
    for task in hot_tasks:
        bug = task.bug
        bug_num = bug.id
        bugs[bug_num] = {}
        bugs[bug_num]['title'] = bug.title
        bugs[bug_num]['heat'] = bug.heat
    return bugs


if __name__ == '__main__':
    lp = LaunchpadService(config={'read_only': False, 'display_name':
                          'foundations-recent-package-bug-tasks'})
    ubuntu = lp.load_project('ubuntu')

    # only show the hottest bugs affecting the dev release or the current
    # stable one
    rtags = [s.name for s in ubuntu.series if s.status in
             ['Active Development', 'Current Stable Release']]

    parser = optparse.OptionParser()
    parser.add_option("--team", help="Team name")

    (opt, args) = parser.parse_args()

    data = []

    json_filename = "%s-hottest-bugs.json" % (opt.team)

    if opt.team:
        team = lp.launchpad.people[opt.team]

    hot_tasks = team.searchTasks(
        tags=rtags, order_by='-heat')[:10]

    bugs = process_tasks(hot_tasks)

    for bug in bugs:
        data.append({'bug': bug,
                     'title': bugs[bug]['title'],
                     'heat': bugs[bug]['heat']})

    json_file = open(json_filename, 'w')
    json_file.write(json.dumps(data, indent=4))
    json_file.close()
