#!/usr/bin/python

from lpltk import LaunchpadService

import datetime
import operator
import optparse
import simplejson as json


def process_tasks(recent_tasks):
    bugs = {}
    for task in recent_tasks:
        if task.bug.duplicate_of:
            master_bug = task.bug.duplicate_of
            master_bug_num = master_bug.id
            if master_bug_num not in bugs.keys():
                bugs[master_bug_num] = {}
            bugs[master_bug_num]['dupes'] = bugs[master_bug_num].setdefault('dupes', 0) + 1
            bugs[master_bug_num]['title'] = master_bug.title
        else:
            continue
    return bugs


if __name__ == '__main__':
    lp = LaunchpadService(config=
        {'read_only': False,
         'display_name': 'foundations-recent-package-bug-tasks'})
    ubuntu = lp.load_project('ubuntu')

    parser = optparse.OptionParser()
    parser.add_option("--within", help="Filter on bugs reported within \
X day(s)")
    parser.add_option("--team", help="Team name")
    parser.add_option("--minimum", help="Minimum number of duplicates \
default of 3", type=int, default=3)

    (opt, args) = parser.parse_args()

    today = datetime.datetime.utcnow()
    periods = opt.within.split(',')

    data = []

    for period in periods:
        json_filename = "%s-recent-duplicate-bugs.json" % (opt.team)

        target_date = today - datetime.timedelta(int(period))

        if opt.team:
            team = lp.launchpad.people[opt.team]

        recent_tasks = ubuntu.searchTasks(created_since=target_date,
            status=['New', 'Incomplete', 'Incomplete (with response)',
                'Incomplete (without response)', 'Confirmed', 'Triaged',
                'In Progress', 'Fix Committed', 'Fix Released', 'Invalid',
                'Opinion', "Won't Fix"],
            bug_supervisor=team, omit_duplicates=False, order_by='targetname')

        bug_freq = process_tasks(recent_tasks)
        bug_totals = {}
        for master_bug in bug_freq.keys():
            bug_totals[master_bug] = bug_freq[master_bug]['dupes']

        sorted_totals = sorted(bug_totals.iteritems(),
                               key=operator.itemgetter(1),
                               reverse=True)

        for key in sorted_totals:
            bug_num = key[0]
            bug = lp.get_bug(bug_num)
            dupe_count = bug.duplicates_count
            if dupe_count < opt.minimum:
                continue
            data.append({'master': bug_num,
                'recent_dupes': bug_freq[bug_num]['dupes'],
                'dupe_count': dupe_count,
                'title': bug_freq[bug_num]['title'],
                'period': period})

    json_file = open(json_filename, 'w')
    json_file.write(json.dumps(data, indent=4))
    json_file.close()
