#!/usr/bin/python

from lpltk import LaunchpadService

import optparse
import simplejson as json


def process_tasks(resolved_upstream):
    #bugs = {number: title, ....}
    bugs = {}
    for task in resolved_upstream:
        release_noted = False
        for rtask in task.related_tasks:
            if rtask.bug_target_name == 'ubuntu-release-notes':
                if rtask.status == 'Fix Released':
                    release_noted = True
        if release_noted:
            continue
        bug_num = task.bug.id
        bugs[bug_num] = {}
        bugs[bug_num]['title'] = task.bug.title
        bugs[bug_num]['watches'] = {}
        # it might be useful to filter on date_last_changed
        for bugwatch in task.bug.bug_watches:
            bugs[bug_num]['watches'][bugwatch.url] = '%s' % bugwatch.date_last_changed
    return bugs


if __name__ == '__main__':
    lp = LaunchpadService(config=
        {'read_only': False,
         'display_name': 'foundations-recent-package-bug-tasks'})
    ubuntu = lp.load_project('ubuntu')

    parser = optparse.OptionParser()
    parser.add_option("--team", help="Team name")

    (opt, args) = parser.parse_args()

    data = []

    json_filename = "%s-resolved-upstream-bugs.json" % (opt.team)

    if opt.team:
        team = lp.launchpad.people[opt.team]

    resolved_tasks = team.searchTasks(
        status_upstream='resolved_upstream',
        order_by='-date_last_updated')[:50]

    bugs = process_tasks(resolved_tasks)

    for bug in bugs:
        data.append({'bug': bug, 
            'title': bugs[bug]['title'],
            'watches': bugs[bug]['watches']})

    json_file = open(json_filename, 'w')
    json_file.write(json.dumps(data, indent=4))
    json_file.close()
