#!/usr/bin/python
#
# Author: Brian Murray <brian@canonical.com>
# Copyright (C) 2011 Canonical, Ltd.
# License: GPLv3
#
# Using a local copy of the UDD (Ultimate Debian Database) to find bug
# reports with the same bug watch since Launchpad doesn't do this easily
# LP: #259194

import lpl_common
import optparse
import psycopg2


def connect_to_db():
    host = 'localhost'
    dbname = 'udd'
    user = 'udd'
    connect = psycopg2.connect("host=%s dbname=%s user=%s password=password" %
                               (host, dbname, user))
    return connect.cursor()

lp = lpl_common.connect(use_edge=False, version='devel')

cur = connect_to_db()

parser = optparse.OptionParser(usage="usage %prog --team TEAM")
parser.add_option('--team',
    help='Display bugs about packages TEAM is subscribed to')
parser.add_option('--package', help='source package name')
(opt, args) = parser.parse_args()

pkgs = None

if opt.package:
    pkgs = opt.package.split(',')

if opt.team:
    team = lp.people[opt.team]
    pkgs = [p.display_name.split()[0] for p in team.getBugSubscriberPackages()]

ubuntu = lp.projects['ubuntu']

cur.execute('''SELECT  DISTINCT(ubuntu_bugs_tasks.bug), ubuntu_bugs_tasks.watch,
                ubuntu_bugs_tasks.package
            FROM ubuntu_bugs_tasks
            INNER JOIN
             (SELECT bug, watch, COUNT(watch) AS numoccurences
              FROM ubuntu_bugs_tasks WHERE watch IS NOT NULL
              GROUP BY bug, watch)
            AS duplicates
            ON ubuntu_bugs_tasks.watch = duplicates.watch
            WHERE ubuntu_bugs_tasks.watch IS NOT NULL
            AND ubuntu_bugs_tasks.bug != duplicates.bug
            ORDER BY ubuntu_bugs_tasks.watch
            ''')
results = cur.fetchall()

for result in results:
    if 'mailto:' in result[1]:
        continue
    if pkgs is not None:
        if result[2] not in pkgs:
            continue
    print 'LP: #%s (%s) - %s' % (result[0], result[2], result[1])
